       PROGRAM WELDTOUGH
C
C copyright H K D H Bhadeshia, University of Cambridge
C November 1994
C
C Based on paper by Bhadeshia, Svensson and MacKay
C Materials Science and Technology, 1995, Vol. 11, p.1046
C
C Program predicts the toughness of manual metal arc and
C  Submerged arc Carbon-Mangenese Steel weld deposits
C  in all weld-metal tests. It does not give error bars,
C  which are discussed in the original paper.
C
C To use the output from neural network analysis 
C  of the toughness of steel weld deposits
C
C The coefficients, in the form of weights are stored in a file
C  called WEIGHTS in unit 4
C
C X contains normalized values of the input variables
C W1 and W2 contain the weights
C THETA1 and THETA2 are the biases
C Y is the output (normalized)
C AMIN and AMAX contain the minimum and maximum unnormalized values of the
C input variables and of the output
C
      IMPLICIT NONE
      DOUBLE PRECISION X(100), W1(10,100), W2(10), H(10)
      DOUBLE PRECISION AMIN(15), AMAX(15), THETA1(100)
      DOUBLE PRECISION A, AVER, THETA2, Y
C
      INTEGER I, IN, IHID, J
C
      OPEN(UNIT=4, FILE='WEIGHTS')
C
C Remember to set matrix dimensions according to number of inputs
C   and number of hidden units. Currently set at 100 and 10 respectively.
C
       AVER = 1.1        
C
       DO 20 I = 1,15
         READ(4,*) AMIN(I),AMAX(I)
20     CONTINUE
C
C Set number of inputs
           IN=14
C
C Set number of hidden units
           IHID=4
C
C Read a set of unnormalized inputs
C
         WRITE(*,101)
101      FORMAT(10X,' PROCESS (-0.5 for MMA, 0.5 for Submerged Arc)?')
         READ(*,*)X(1)
         WRITE(*,102)
102      FORMAT(10X,' Yield Strength at 20 Centigrade, MPa  (347-645)?')
         READ(*,*)X(2)
         WRITE(*,103)
103      FORMAT(10X,' Carbon, wt.% (0.029-0.130)?')
         READ(*,*)X(3)
         WRITE(*,104)
104      FORMAT(10X,' Silicon, wt.% (0.28-1.14)?')
         READ(*,*)X(4)
         WRITE(*,105)
105      FORMAT(10X,' Manganese, wt.% (0.77-2.50) ?')
         READ(*,*)X(5)
         WRITE(*,106)
106      FORMAT(10X,' Phosphorus, wt.% (0.008-0.028)?')
         READ(*,*)X(6)
         WRITE(*,107)
107      FORMAT(10X,' Sulphur, wt.% (0.002-0.017)?')
         READ(*,*)X(7)
         WRITE(*,108)
108      FORMAT(10X,' Aluminium, wt.% (0.001-0.040)?')
         READ(*,*)X(8)
         WRITE(*,109)
109      FORMAT(10X,' Nitrogen, ppmw (26-119)?')
         READ(*,*)X(9)
         WRITE(*,110)
110      FORMAT(10X,' Oxygen, ppmw (234-821)?')
         READ(*,*)X(10)
         WRITE(*,111)
111      FORMAT(10X,' (Primary Microstructure % (0-91)?')
         READ(*,*)X(11)
         WRITE(*,112)
112      FORMAT(10X,' Allotriomorphic Ferrite % (16-62)?')
         READ(*,*)X(12)
         WRITE(*,113)
113      FORMAT(10X,' Acicular Ferrite %(11-81)?')
         READ(*,*)X(13)
         WRITE(*,114)
114      FORMAT(10X,' Temperature, Centigrade (-60 to 20) ?')
         READ(*,*)X(14)
C
         CALL WRITT(X)
C
C
         X(14) = X(14)+273.00d+00
C
         DO 1 I = 1,IN
          IF (I .GT. 1) THEN
             X(I) = ((X(I) -AMIN(I))/(AMAX(I)-AMIN(I)))-0.5D+00
          ENDIF
1        CONTINUE
C  
C Read all weights, and theta parameters
C
          DO 2 I = 1,IHID
            READ(4,*) THETA1(I)
C           WRITE(*,91) THETA1(I)
              DO 3, J = 1,IN
                 READ(4,*) W1(I,J)
C                WRITE(*,91) W1(I,J)
3           CONTINUE
2         CONTINUE
C 
C        
          READ(4,*) THETA2
C         WRITE(*,91) THETA2
          DO 21 I=1,IHID
            READ(4,*) W2(I)
C           WRITE(*,91) W2(I)
21        CONTINUE
C
C
C Do calculation of the normalized output value for the given set of
C  normalized input     
C--------------------------------
C First equation
C
       DO 6, I = 1,IHID
          A = 0.0D+00
C
               DO 5 J=1,IN
                 A = A+W1(I,J)*X(J)
5              CONTINUE
C
          H(I) = DTANH(A+THETA1(I))
6      CONTINUE
C
C--------------------------------
C Second equation
C
       A = 0.0D+00
       DO 7, I = 1,IHID
          A = A+W2(I)*H(I)
7      CONTINUE
C 
       Y = A+THETA2     
       Y = (Y+0.5)*(AMAX(15)-AMIN(15))+AMIN(15)      
C      
       WRITE(*,91) Y
C
       CLOSE(4)
C
       STOP
90     FORMAT(F10.2)
91     FORMAT(//10X,' Charpy Toughness, Joules = 'F8.0)
       END
C
C******************************************************************
C
      SUBROUTINE WRITT(A)
      IMPLICIT NONE
      DOUBLE PRECISION A(100)
      WRITE(*,4) A(1),A(2),A(3),A(4), A(5),A(6),A(7),A(8),A(9),
     &   A(10),A(11),A(12),A(13),A(14)
C
C
C View input variables 

4      FORMAT(//////
     &20X,'      ** WELD CHARPY TOUGHNESS **           '//
     &20X,'Darwin College, Cambridge & ESAB AB (Sweden)'/
     &20X,'       Bhadeshia, Svensson & MacKay      ' ///
     &10X,' Process     ',F6.1,'      ',
     &9X,' Yield Strength     ',F6.0,'  MPa '/
     &10X,' Carbon      ',F6.3,'  wt.%',
     &9X,' Silicon           ',F6.3,'  wt.%'/
     &10X,' Manganese   ',F6.3,'  wt.%',9X,
     &' Phosphorus        ',F6.3,'  wt.%'/
     &10X,' Sulphur     ',F6.3,'  wt.%',
     &9X,' Aluminium         ',F6.3,'  wt.%'/
     &10X,' Nitrogen    ',F6.0,'  ppmw',9X,
     &' Oxygen            ',F6.0,'  ppmw'/
     &10X,' Primary     ',F6.0,'  %   ',9X,
     &' Allotriomorphic   ',F6.0,'  %   '/
     &10X,' Acicular    ',F6.0,'  %   ',9X,
     &' Temperature       ',F6.0,'  C   '/
     &/)
      RETURN
      END
C
C********************************************************************
